@@ -1,4 +1,4 @@
-/* $Id: cmd-clock-mode.c,v 1.7 2009/11/14 17:56:39 tcunha Exp $ */
+/* $Id: cmd-clock-mode.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,22 +28,21 @@ int	cmd_clock_mode_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_clock_mode_entry = {
 	"clock-mode", NULL,
+	"t:", 0, 0,
 	CMD_TARGET_PANE_USAGE,
-	0, "",
-	cmd_target_init,
-	cmd_target_parse,
-	cmd_clock_mode_exec,
-	cmd_target_free,
-	cmd_target_print
+	0,
+	NULL,
+	NULL,
+	cmd_clock_mode_exec
 };
 
 int
 cmd_clock_mode_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct cmd_target_data	*data = self->data;
+	struct args		*args = self->args;
 	struct window_pane	*wp;
 
-	if (cmd_find_pane(ctx, data->target, NULL, &wp) == NULL)
+	if (cmd_find_pane(ctx, args_get(args, 't'), NULL, &wp) == NULL)
 		return (-1);
 
 	window_pane_set_mode(wp, &window_clock_mode);
