@@ -1,4 +1,4 @@
-/* $Id: cmd-confirm-before.c,v 1.12 2009/11/14 17:56:39 tcunha Exp $ */
+/* $Id: cmd-confirm-before.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2009 Tiago Cunha <me@tiagocunha.org>
@@ -25,21 +25,20 @@
  * Asks for confirmation before executing a command.
  */
 
+void	cmd_confirm_before_key_binding(struct cmd *, int);
 int	cmd_confirm_before_exec(struct cmd *, struct cmd_ctx *);
-void	cmd_confirm_before_init(struct cmd *, int);
 
 int	cmd_confirm_before_callback(void *, const char *);
 void	cmd_confirm_before_free(void *);
 
 const struct cmd_entry cmd_confirm_before_entry = {
 	"confirm-before", "confirm",
-	CMD_TARGET_CLIENT_USAGE " command",
-	CMD_ARG1, "",
-	cmd_confirm_before_init,
-	cmd_target_parse,
-	cmd_confirm_before_exec,
-	cmd_target_free,
-	cmd_target_print
+	"p:t:", 1, 1,
+	"[-p prompt] " CMD_TARGET_CLIENT_USAGE " command",
+	0,
+	cmd_confirm_before_key_binding,
+	NULL,
+	cmd_confirm_before_exec
 };
 
 struct cmd_confirm_before_data {
@@ -48,19 +47,19 @@ struct cmd_confirm_before_data {
 };
 
 void
-cmd_confirm_before_init(struct cmd *self, int key)
+cmd_confirm_before_key_binding(struct cmd *self, int key)
 {
-	struct cmd_target_data	*data;
-
-	cmd_target_init(self, key);
-	data = self->data;
-
 	switch (key) {
 	case '&':
-		data->arg = xstrdup("kill-window");
+		self->args = args_create(1, "kill-window");
+		args_set(self->args, 'p', "kill-window #W? (y/n)");
 		break;
 	case 'x':
-		data->arg = xstrdup("kill-pane");
+		self->args = args_create(1, "kill-pane");
+		args_set(self->args, 'p', "kill-pane #P? (y/n)");
+		break;
+	default:
+		self->args = args_create(0);
 		break;
 	}
 }
@@ -68,33 +67,37 @@ cmd_confirm_before_init(struct cmd *self, int key)
 int
 cmd_confirm_before_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct cmd_target_data		*data = self->data;
+	struct args			*args = self->args;
 	struct cmd_confirm_before_data	*cdata;
 	struct client			*c;
-	char				*buf, *cmd, *ptr;
+	char				*cmd, *copy, *new_prompt, *ptr;
+	const char			*prompt;
 
 	if (ctx->curclient == NULL) {
 		ctx->error(ctx, "must be run interactively");
 		return (-1);
 	}
 
-	if ((c = cmd_find_client(ctx, data->target)) == NULL)
+	if ((c = cmd_find_client(ctx, args_get(args, 't'))) == NULL)
 		return (-1);
 
-	ptr = xstrdup(data->arg);
-	if ((cmd = strtok(ptr, " \t")) == NULL)
-		cmd = ptr;
-	xasprintf(&buf, "Confirm '%s'? (y/n) ", cmd);
-	xfree(ptr);
+	if ((prompt = args_get(args, 'p')) != NULL)
+		xasprintf(&new_prompt, "%s ", prompt);
+	else {
+		ptr = copy = xstrdup(args->argv[0]);
+		cmd = strsep(&ptr, " \t");
+		xasprintf(&new_prompt, "Confirm '%s'? (y/n) ", cmd);
+		xfree(copy);
+	}
 
 	cdata = xmalloc(sizeof *cdata);
-	cdata->cmd = xstrdup(data->arg);
+	cdata->cmd = xstrdup(args->argv[0]);
 	cdata->c = c;
-	status_prompt_set(cdata->c, buf,
+	status_prompt_set(cdata->c, new_prompt, NULL,
 	    cmd_confirm_before_callback, cmd_confirm_before_free, cdata,
 	    PROMPT_SINGLE);
 
-	xfree(buf);
+	xfree(new_prompt);
 	return (1);
 }
 
