@@ -1,4 +1,4 @@
-/* $Id: cmd-copy-mode.c,v 1.28 2010/08/11 22:18:28 tcunha Exp $ */
+/* $Id: cmd-copy-mode.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -24,48 +24,40 @@
  * Enter copy mode.
  */
 
-void	cmd_copy_mode_init(struct cmd *, int);
+void	cmd_copy_mode_key_binding(struct cmd *, int);
 int	cmd_copy_mode_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_copy_mode_entry = {
 	"copy-mode", NULL,
+	"t:u", 0, 0,
 	"[-u] " CMD_TARGET_PANE_USAGE,
-	0, "u",
-	cmd_copy_mode_init,
-	cmd_target_parse,
-	cmd_copy_mode_exec,
-	cmd_target_free,
-	cmd_target_print
+	0,
+	cmd_copy_mode_key_binding,
+	NULL,
+	cmd_copy_mode_exec
 };
 
 void
-cmd_copy_mode_init(struct cmd *self, int key)
+cmd_copy_mode_key_binding(struct cmd *self, int key)
 {
-	struct cmd_target_data	*data;
-
-	cmd_target_init(self, key);
-	data = self->data;
-
-	switch (key) {
-	case KEYC_PPAGE:
-		cmd_set_flag(&data->chflags, 'u');
-		break;
-	}
+	self->args = args_create(0);
+	if (key == KEYC_PPAGE)
+		args_set(self->args, 'u', NULL);
 }
 
 int
 cmd_copy_mode_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct cmd_target_data	*data = self->data;
+	struct args		*args = self->args;
 	struct window_pane	*wp;
 
-	if (cmd_find_pane(ctx, data->target, NULL, &wp) == NULL)
+	if (cmd_find_pane(ctx, args_get(args, 't'), NULL, &wp) == NULL)
 		return (-1);
 
 	if (window_pane_set_mode(wp, &window_copy_mode) != 0)
 		return (0);
 	window_copy_init_from_pane(wp);
-	if (wp->mode == &window_copy_mode && cmd_check_flag(data->chflags, 'u'))
+	if (wp->mode == &window_copy_mode && args_has(self->args, 'u'))
 		window_copy_pageup(wp);
 
 	return (0);
