@@ -1,4 +1,4 @@
-/* $Id: cmd-list-clients.c,v 1.20 2009/11/28 14:50:36 tcunha Exp $ */
+/* $Id: cmd-list-clients.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -31,23 +31,31 @@ int	cmd_list_clients_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_list_clients_entry = {
 	"list-clients", "lsc",
-	"",
-	0, "",
+	"t:", 0, 0,
+	CMD_TARGET_SESSION_USAGE,
+	0,
 	NULL,
 	NULL,
-	cmd_list_clients_exec,
-	NULL,
-	NULL
+	cmd_list_clients_exec
 };
 
 /* ARGSUSED */
 int
-cmd_list_clients_exec(unused struct cmd *self, struct cmd_ctx *ctx)
+cmd_list_clients_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
+	struct args 	*args = self->args;
 	struct client	*c;
+	struct session  *s;
 	u_int		 i;
 	const char	*s_utf8;
 
+	if (args_has(args, 't')) {
+		s = cmd_find_session(ctx, args_get(args, 't'), 0);
+		if (s == NULL)
+			return (-1);
+	} else
+		s = NULL;
+
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
 		c = ARRAY_ITEM(&clients, i);
 		if (c == NULL || c->session == NULL)
@@ -57,6 +65,9 @@ cmd_list_clients_exec(unused struct cmd *self, struct cmd_ctx *ctx)
 			s_utf8 = " (utf8)";
 		else
 			s_utf8 = "";
+
+		if (s != NULL && s != c->session)
+			continue;
 		ctx->print(ctx, "%s: %s [%ux%u %s]%s", c->tty.path,
 		    c->session->name, c->tty.sx, c->tty.sy,
 		    c->tty.termname, s_utf8);
