@@ -1,4 +1,4 @@
-/* $Id: cmd-list-keys.c,v 1.25 2010/10/24 01:31:57 tcunha Exp $ */
+/* $Id: cmd-list-keys.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -32,37 +32,43 @@ int	cmd_list_keys_table(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_list_keys_entry = {
 	"list-keys", "lsk",
+	"t:", 0, 0,
 	"[-t key-table]",
-	0, "",
-	cmd_target_init,
-	cmd_target_parse,
-	cmd_list_keys_exec,
-	cmd_target_free,
-	cmd_target_print
+	0,
+	NULL,
+	NULL,
+	cmd_list_keys_exec
 };
 
 int
 cmd_list_keys_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct cmd_target_data	*data = self->data;
+	struct args		*args = self->args;
 	struct key_binding	*bd;
 	const char		*key;
-	char			 tmp[BUFSIZ];
+	char			 tmp[BUFSIZ], flags[8];
 	size_t			 used;
 	int			 width, keywidth;
 
-	if (data->target != NULL)
+	if (args_has(args, 't'))
 		return (cmd_list_keys_table(self, ctx));
 
 	width = 0;
+	*flags = '\0';
+
 	SPLAY_FOREACH(bd, key_bindings, &key_bindings) {
 		key = key_string_lookup_key(bd->key & ~KEYC_PREFIX);
 		if (key == NULL)
 			continue;
 
-		keywidth = strlen(key) + 1;
-		if (!(bd->key & KEYC_PREFIX))
-			keywidth += 2;
+		keywidth = strlen(key);
+		if (!(bd->key & KEYC_PREFIX)) {
+			if (bd->can_repeat)
+				keywidth += 4;
+			else
+				keywidth += 3;
+		} else if (bd->can_repeat)
+			keywidth += 3;
 		if (keywidth > width)
 			width = keywidth;
 	}
@@ -71,22 +77,22 @@ cmd_list_keys_exec(struct cmd *self, struct cmd_ctx *ctx)
 		key = key_string_lookup_key(bd->key & ~KEYC_PREFIX);
 		if (key == NULL)
 			continue;
-		used = xsnprintf(tmp, sizeof tmp, "%*s: ", width, key);
+
+		if (!(bd->key & KEYC_PREFIX)) {
+			if (bd->can_repeat)
+				xsnprintf(flags, sizeof flags, "-rn ");
+			else
+				xsnprintf(flags, sizeof flags, "-n ");
+		} else if (bd->can_repeat)
+			xsnprintf(flags, sizeof flags, "-r ");
+
+		used = xsnprintf(tmp, sizeof tmp, "%s%*s ",
+		    flags, (int) (width - strlen(flags)), key);
 		if (used >= sizeof tmp)
 			continue;
 
-		if (!(bd->key & KEYC_PREFIX)) {
-			used = strlcat(tmp, "(no prefix) ", sizeof tmp);
-			if (used >= sizeof tmp)
-				continue;
-		}
-		if (bd->can_repeat) {
-			used = strlcat(tmp, "(repeat) ", sizeof tmp);
-			if (used >= sizeof tmp)
-				continue;
-		}
 		cmd_list_print(bd->cmdlist, tmp + used, (sizeof tmp) - used);
-		ctx->print(ctx, "%s", tmp);
+		ctx->print(ctx, "bind-key %s", tmp);
 	}
 
 	return (0);
@@ -95,24 +101,30 @@ cmd_list_keys_exec(struct cmd *self, struct cmd_ctx *ctx)
 int
 cmd_list_keys_table(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct cmd_target_data		*data = self->data;
+	struct args			*args = self->args;
+	const char			*tablename;
 	const struct mode_key_table	*mtab;
 	struct mode_key_binding		*mbind;
 	const char			*key, *cmdstr, *mode;
-	int			 	 width, keywidth;
+	int			 	 width, keywidth, any_mode;
 
-	if ((mtab = mode_key_findtable(data->target)) == NULL) {
-		ctx->error(ctx, "unknown key table: %s", data->target);
+	tablename = args_get(args, 't');
+	if ((mtab = mode_key_findtable(tablename)) == NULL) {
+		ctx->error(ctx, "unknown key table: %s", tablename);
 		return (-1);
 	}
 
 	width = 0;
+	any_mode = 0;
 	SPLAY_FOREACH(mbind, mode_key_tree, mtab->tree) {
 		key = key_string_lookup_key(mbind->key);
 		if (key == NULL)
 			continue;
 
-		keywidth = strlen(key) + 1;
+		if (mbind->mode != 0)
+			any_mode = 1;
+
+		keywidth = strlen(key);
 		if (keywidth > width)
 			width = keywidth;
 	}
@@ -124,10 +136,13 @@ cmd_list_keys_table(struct cmd *self, struct cmd_ctx *ctx)
 
 		mode = "";
 		if (mbind->mode != 0)
-			mode = "(command mode) ";
+			mode = "c";
 		cmdstr = mode_key_tostring(mtab->cmdstr, mbind->cmd);
-		if (cmdstr != NULL)
-			ctx->print(ctx, "%*s: %s%s", width, key, mode, cmdstr);
+		if (cmdstr != NULL) {
+			ctx->print(ctx, "bind-key -%st %s%s %*s %s",
+			    mode, any_mode && *mode == '\0' ? " " : "",
+			    mtab->name, (int) width, key, cmdstr);
+		}
 	}
 
 	return (0);
