@@ -1,4 +1,4 @@
-/* $Id: cmd-list-sessions.c,v 1.26 2010/12/22 15:36:44 tcunha Exp $ */
+/* $Id: cmd-list-sessions.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -30,13 +30,13 @@
 int	cmd_list_sessions_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_list_sessions_entry = {
-	"list-sessions", "ls", "",
-	0, "",
+	"list-sessions", "ls",
+	"", 0, 0,
+	"",
+	0,
 	NULL,
 	NULL,
-	cmd_list_sessions_exec,
-	NULL,
-	NULL
+	cmd_list_sessions_exec
 };
 
 /* ARGSUSED */
