@@ -1,4 +1,4 @@
-/* $Id: cmd-list-windows.c,v 1.44 2010/12/06 21:56:32 nicm Exp $ */
+/* $Id: cmd-list-windows.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,35 +28,65 @@
 
 int	cmd_list_windows_exec(struct cmd *, struct cmd_ctx *);
 
+void	cmd_list_windows_server(struct cmd_ctx *);
+void	cmd_list_windows_session(struct session *, struct cmd_ctx *, int);
+
 const struct cmd_entry cmd_list_windows_entry = {
 	"list-windows", "lsw",
-	CMD_TARGET_SESSION_USAGE,
-	0, "",
-	cmd_target_init,
-	cmd_target_parse,
-	cmd_list_windows_exec,
-	cmd_target_free,
-	cmd_target_print
+	"at:", 0, 0,
+	"[-a] " CMD_TARGET_SESSION_USAGE,
+	0,
+	NULL,
+	NULL,
+	cmd_list_windows_exec
 };
 
 int
 cmd_list_windows_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct cmd_target_data	*data = self->data;
-	struct session		*s;
-	struct winlink		*wl;
-	char			*layout;
+	struct args	*args = self->args;
+	struct session	*s;
 
-	if ((s = cmd_find_session(ctx, data->target)) == NULL)
-		return (-1);
-
-	RB_FOREACH(wl, winlinks, &s->windows) {
-		layout = layout_dump(wl->window);
-		ctx->print(ctx, "%d: %s [%ux%u] [layout %s]%s",
-		    wl->idx, wl->window->name, wl->window->sx, wl->window->sy,
-		    layout, wl == s->curw ? " (active)" : "");
-		xfree(layout);
+	if (args_has(args, 'a'))
+		cmd_list_windows_server(ctx);
+	else {
+		s = cmd_find_session(ctx, args_get(args, 't'), 0);
+		if (s == NULL)
+			return (-1);
+		cmd_list_windows_session(s, ctx, 0);
 	}
 
 	return (0);
 }
+
+void
+cmd_list_windows_server(struct cmd_ctx *ctx)
+{
+	struct session	*s;
+
+	RB_FOREACH(s, sessions, &sessions)
+		cmd_list_windows_session(s, ctx, 1);
+}
+
+void
+cmd_list_windows_session(struct session *s, struct cmd_ctx *ctx, int type)
+{
+	struct winlink	*wl;
+	char		*layout;
+
+	RB_FOREACH(wl, winlinks, &s->windows) {
+		layout = layout_dump(wl->window);
+		if (type) {
+			ctx->print(ctx, "%s:%d: %s [%ux%u] [layout %s]%s",
+			    s->name, wl->idx, wl->window->name, wl->window->sx,
+			    wl->window->sy, layout,
+			    wl == s->curw ? " (active)" : "");
+		} else {
+			ctx->print(ctx, "%d: %s [%ux%u] [layout %s]%s",
+			    wl->idx, wl->window->name, wl->window->sx,
+			    wl->window->sy, layout,
+			    wl == s->curw ? " (active)" : "");
+		}
+		xfree(layout);
+	}
+}
