@@ -1,4 +1,4 @@
-/* $Id: cmd-load-buffer.c,v 1.18 2010/12/22 15:28:50 tcunha Exp $ */
+/* $Id: cmd-load-buffer.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2009 Tiago Cunha <me@tiagocunha.org>
@@ -35,55 +35,57 @@ void	cmd_load_buffer_callback(struct client *, void *);
 
 const struct cmd_entry cmd_load_buffer_entry = {
 	"load-buffer", "loadb",
-	CMD_BUFFER_SESSION_USAGE " path",
-	CMD_ARG1, "",
-	cmd_buffer_init,
-	cmd_buffer_parse,
-	cmd_load_buffer_exec,
-	cmd_buffer_free,
-	cmd_buffer_print
-};
-
-struct cmd_load_buffer_cdata {
-	struct session	*session;
-	int		 buffer;
+	"b:", 1, 1,
+	CMD_BUFFER_USAGE " path",
+	0,
+	NULL,
+	NULL,
+	cmd_load_buffer_exec
 };
 
 int
 cmd_load_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct cmd_buffer_data		*data = self->data;
-	struct cmd_load_buffer_cdata	*cdata;
-	struct session			*s;
-	struct client			*c = ctx->cmdclient;
-	FILE				*f;
-	char		      		*pdata, *new_pdata;
-	size_t				 psize;
-	u_int				 limit;
-	int				 ch;
+	struct args	*args = self->args;
+	struct client	*c = ctx->cmdclient;
+	FILE		*f;
+	const char	*path;
+	char		*pdata, *new_pdata, *cause;
+	size_t		 psize;
+	u_int		 limit;
+	int		 ch, buffer;
+	int		*buffer_ptr;
 
-	if ((s = cmd_find_session(ctx, data->target)) == NULL)
-		return (-1);
+	if (!args_has(args, 'b'))
+		buffer = -1;
+	else {
+		buffer = args_strtonum(args, 'b', 0, INT_MAX, &cause);
+		if (cause != NULL) {
+			ctx->error(ctx, "buffer %s", cause);
+			xfree(cause);
+			return (-1);
+		}
+	}
 
-	if (strcmp(data->arg, "-") == 0) {
+	path = args->argv[0];
+	if (strcmp(path, "-") == 0) {
 		if (c == NULL) {
-			ctx->error(ctx, "%s: can't read from stdin", data->arg);
+			ctx->error(ctx, "%s: can't read from stdin", path);
 			return (-1);
 		}
 		if (c->flags & CLIENT_TERMINAL) {
-			ctx->error(ctx, "%s: stdin is a tty", data->arg);
+			ctx->error(ctx, "%s: stdin is a tty", path);
 			return (-1);
 		}
 		if (c->stdin_fd == -1) {
-			ctx->error(ctx, "%s: can't read from stdin", data->arg);
+			ctx->error(ctx, "%s: can't read from stdin", path);
 			return (-1);
 		}
 
-		cdata = xmalloc(sizeof *cdata);
-		cdata->session = s;
-		cdata->session->references++;
-		cdata->buffer = data->buffer;
-		c->stdin_data = cdata;
+		buffer_ptr = xmalloc(sizeof *buffer_ptr);
+		*buffer_ptr = buffer;
+
+		c->stdin_data = buffer_ptr;
 		c->stdin_callback = cmd_load_buffer_callback;
 
 		c->references++;
@@ -91,8 +93,8 @@ cmd_load_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 		return (1);
 	}
 
-	if ((f = fopen(data->arg, "rb")) == NULL) {
-		ctx->error(ctx, "%s: %s", data->arg, strerror(errno));
+	if ((f = fopen(path, "rb")) == NULL) {
+		ctx->error(ctx, "%s: %s", path, strerror(errno));
 		return (-1);
 	}
 
@@ -108,22 +110,21 @@ cmd_load_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 		pdata[psize++] = ch;
 	}
 	if (ferror(f)) {
-		ctx->error(ctx, "%s: read error", data->arg);
+		ctx->error(ctx, "%s: read error", path);
 		goto error;
 	}
 	if (pdata != NULL)
 		pdata[psize] = '\0';
 
 	fclose(f);
-	f = NULL;
 
-	limit = options_get_number(&s->options, "buffer-limit");
-	if (data->buffer == -1) {
-		paste_add(&s->buffers, pdata, psize, limit);
+	limit = options_get_number(&global_options, "buffer-limit");
+	if (buffer == -1) {
+		paste_add(&global_buffers, pdata, psize, limit);
 		return (0);
 	}
-	if (paste_replace(&s->buffers, data->buffer, pdata, psize) != 0) {
-		ctx->error(ctx, "no buffer %d", data->buffer);
+	if (paste_replace(&global_buffers, buffer, pdata, psize) != 0) {
+		ctx->error(ctx, "no buffer %d", buffer);
 		return (-1);
 	}
 
@@ -140,11 +141,10 @@ error:
 void
 cmd_load_buffer_callback(struct client *c, void *data)
 {
-	struct cmd_load_buffer_cdata	*cdata = data;
-	struct session			*s = cdata->session;
-	char				*pdata;
-	size_t				 psize;
-	u_int				 limit;
+	int	*buffer = data;
+	char	*pdata;
+	size_t	 psize;
+	u_int	 limit;
 
 	/*
 	 * Event callback has already checked client is not dead and reduced
@@ -152,34 +152,23 @@ cmd_load_buffer_callback(struct client *c, void *data)
 	 */
 	c->flags |= CLIENT_EXIT;
 
-	/* Does the target session still exist? */
-	if (!session_alive(s))
-		goto out;
-
 	psize = EVBUFFER_LENGTH(c->stdin_event->input);
-	if (psize == 0)
-		goto out;
-
-	pdata = malloc(psize + 1);
-	if (pdata == NULL)
-		goto out;
+	if (psize == 0 || (pdata = malloc(psize + 1)) == NULL) {
+		xfree(data);
+		return;
+	}
 	bufferevent_read(c->stdin_event, pdata, psize);
 	pdata[psize] = '\0';
 
-	limit = options_get_number(&s->options, "buffer-limit");
-	if (cdata->buffer == -1) {
-		paste_add(&s->buffers, pdata, psize, limit);
-		goto out;
-	}
-	if (paste_replace(&s->buffers, cdata->buffer, pdata, psize) != 0) {
+	limit = options_get_number(&global_options, "buffer-limit");
+	if (*buffer == -1)
+		paste_add(&global_buffers, pdata, psize, limit);
+	else if (paste_replace(&global_buffers, *buffer, pdata, psize) != 0) {
 		/* No context so can't use server_client_msg_error. */
 		evbuffer_add_printf(
-		    c->stderr_event->output, "no buffer %d\n", cdata->buffer);
+		    c->stderr_event->output, "no buffer %d\n", *buffer);
 		bufferevent_enable(c->stderr_event, EV_WRITE);
-		goto out;
 	}
 
-out:
-	cdata->session->references--;
-	xfree(cdata);
+	xfree(data);
 }
