@@ -1,4 +1,4 @@
-/* $Id: cmd-lock-server.c,v 1.9 2009/11/28 14:50:36 tcunha Exp $ */
+/* $Id: cmd-lock-server.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2008 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -25,27 +25,60 @@
 #include "tmux.h"
 
 /*
- * Lock server.
+ * Lock commands.
  */
 
 int	cmd_lock_server_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_lock_server_entry = {
 	"lock-server", "lock",
+	"", 0, 0,
 	"",
-	0, "",
+	0,
 	NULL,
 	NULL,
-	cmd_lock_server_exec,
+	cmd_lock_server_exec
+};
+
+const struct cmd_entry cmd_lock_session_entry = {
+	"lock-session", "locks",
+	"t:", 0, 0,
+	CMD_TARGET_SESSION_USAGE,
+	0,
 	NULL,
 	NULL,
+	cmd_lock_server_exec
+};
+
+const struct cmd_entry cmd_lock_client_entry = {
+	"lock-client", "lockc",
+	"t:", 0, 0,
+	CMD_TARGET_CLIENT_USAGE,
+	0,
+	NULL,
+	NULL,
+	cmd_lock_server_exec
 };
 
 /* ARGSUSED */
 int
-cmd_lock_server_exec(unused struct cmd *self, unused struct cmd_ctx *ctx)
+cmd_lock_server_exec(struct cmd *self, unused struct cmd_ctx *ctx)
 {
-	server_lock();
+	struct args	*args = self->args;
+	struct client	*c;
+	struct session	*s;
+
+	if (self->entry == &cmd_lock_server_entry)
+		server_lock();
+	else if (self->entry == &cmd_lock_session_entry) {
+		if ((s = cmd_find_session(ctx, args_get(args, 't'), 0)) == NULL)
+			return (-1);
+		server_lock_session(s);
+	} else {
+		if ((c = cmd_find_client(ctx, args_get(args, 't'))) == NULL)
+			return (-1);
+		server_lock_client(c);
+	}
 	recalculate_sizes();
 
 	return (0);
