@@ -1,4 +1,4 @@
-/* $Id: cmd-send-prefix.c,v 1.29 2010/05/22 21:56:04 micahcowan Exp $ */
+/* $Id: cmd-send-prefix.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,24 +28,23 @@ int	cmd_send_prefix_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_send_prefix_entry = {
 	"send-prefix", NULL,
+	"t:", 0, 0,
 	CMD_TARGET_PANE_USAGE,
-	0, "",
-	cmd_target_init,
-	cmd_target_parse,
-	cmd_send_prefix_exec,
-	cmd_target_free,
-	cmd_target_print
+	0,
+	NULL,
+	NULL,
+	cmd_send_prefix_exec
 };
 
 int
 cmd_send_prefix_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct cmd_target_data	*data = self->data;
+	struct args		*args = self->args;
 	struct session		*s;
 	struct window_pane	*wp;
 	struct keylist		*keylist;
 
-	if (cmd_find_pane(ctx, data->target, &s, &wp) == NULL)
+	if (cmd_find_pane(ctx, args_get(args, 't'), &s, &wp) == NULL)
 		return (-1);
 
 	keylist = options_get_data(&s->options, "prefix");
