@@ -1,4 +1,4 @@
-/* $Id: cmd-show-buffer.c,v 1.12 2009/12/04 22:14:47 tcunha Exp $ */
+/* $Id: cmd-show-buffer.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -28,39 +28,47 @@ int	cmd_show_buffer_exec(struct cmd *, struct cmd_ctx *);
 
 const struct cmd_entry cmd_show_buffer_entry = {
 	"show-buffer", "showb",
-	CMD_BUFFER_SESSION_USAGE,
-	0, "",
-	cmd_buffer_init,
-	cmd_buffer_parse,
-	cmd_show_buffer_exec,
-	cmd_buffer_free,
-	cmd_buffer_print
+	"b:", 0, 0,
+	CMD_BUFFER_USAGE,
+	0,
+	NULL,
+	NULL,
+	cmd_show_buffer_exec
 };
 
 int
 cmd_show_buffer_exec(struct cmd *self, struct cmd_ctx *ctx)
 {
-	struct cmd_buffer_data	*data = self->data;
+	struct args		*args = self->args;
 	struct session		*s;
 	struct paste_buffer	*pb;
-	char			*in, *buf, *ptr;
+	int			 buffer;
+	char			*in, *buf, *ptr, *cause;
 	size_t			 size, len;
 	u_int			 width;
 
-	if ((s = cmd_find_session(ctx, data->target)) == NULL)
+	if ((s = cmd_find_session(ctx, NULL, 0)) == NULL)
 		return (-1);
 
-	if (data->buffer == -1) {
-		if ((pb = paste_get_top(&s->buffers)) == NULL) {
+	if (!args_has(args, 'b')) {
+		if ((pb = paste_get_top(&global_buffers)) == NULL) {
 			ctx->error(ctx, "no buffers");
 			return (-1);
 		}
-	} else if ((pb = paste_get_index(&s->buffers, data->buffer)) == NULL) {
-		ctx->error(ctx, "no buffer %d", data->buffer);
-		return (-1);
+	} else {
+		buffer = args_strtonum(args, 'b', 0, INT_MAX, &cause);
+		if (cause != NULL) {
+			ctx->error(ctx, "buffer %s", cause);
+			xfree(cause);
+			return (-1);
+		}
+
+		pb = paste_get_index(&global_buffers, buffer);
+		if (pb == NULL) {
+			ctx->error(ctx, "no buffer %d", buffer);
+			return (-1);
+		}
 	}
-	if (pb == NULL)
-		return (0);
 
 	size = pb->size;
 	if (size > SIZE_MAX / 4 - 1)
