@@ -1,4 +1,4 @@
-/* $Id: osdep-netbsd.c,v 1.9 2009/09/24 12:30:22 nicm Exp $ */
+/* $Id: osdep-netbsd.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -22,6 +22,7 @@
 #include <sys/sysctl.h>
 
 #include <errno.h>
+#include <event.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
@@ -33,6 +34,7 @@
 
 struct kinfo_proc2	*cmp_procs(struct kinfo_proc2 *, struct kinfo_proc2 *);
 char			*osdep_get_name(int, char *);
+struct event_base	*osdep_event_init(void);
 
 struct kinfo_proc2 *
 cmp_procs(struct kinfo_proc2 *p1, struct kinfo_proc2 *p2)
@@ -120,3 +122,9 @@ error:
 	free(buf);
 	return (NULL);
 }
+
+struct event_base *
+osdep_event_init(void)
+{
+	return (event_init());
+}
