@@ -1,4 +1,4 @@
-/* $Id: osdep-sunos.c,v 1.2 2009/10/15 07:11:25 nicm Exp $ */
+/* $Id: osdep-sunos.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2009 Todd Carson <toc@daybefore.net>
@@ -19,6 +19,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 
+#include <event.h>
 #include <fcntl.h>
 #include <procfs.h>
 #include <stdio.h>
@@ -63,3 +64,9 @@ osdep_get_name(int fd, char *tty)
 
 	return (xstrdup(p.pr_fname));
 }
+
+struct event_base *
+osdep_event_init(void)
+{
+	return (event_init());
+}
