@@ -1,4 +1,4 @@
-/* $Id: osdep-unknown.c,v 1.5 2009/04/29 23:07:35 nicm Exp $ */
+/* $Id: osdep-unknown.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -18,6 +18,8 @@
 
 #include <sys/types.h>
 
+#include <event.h>
+
 #include "tmux.h"
 
 char *
@@ -25,3 +27,9 @@ osdep_get_name(unused int fd, unused char *tty)
 {
 	return (NULL);
 }
+
+struct event_base *
+osdep_event_init(void)
+{
+	return (event_init());
+}
