@@ -1,4 +1,4 @@
-/* $Id: screen-redraw.c,v 1.53 2010/09/18 15:43:53 tcunha Exp $ */
+/* $Id: screen-redraw.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -176,6 +176,10 @@ screen_redraw_screen(struct client *c, int status_only, int borders_only)
 	u_int		 	 i, j, type;
 	int		 	 status, fg, bg;
 
+	/* Suspended clients should not be updated. */
+	if (c->flags & CLIENT_SUSPENDED)
+		return;
+
 	/* Get status line, er, status. */
 	if (c->message_string != NULL || c->prompt_string != NULL)
 		status = 1;
