@@ -1,4 +1,4 @@
-/* $Id: screen-write.c,v 1.90 2010/06/16 18:09:23 micahcowan Exp $ */
+/* $Id: screen-write.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -829,16 +829,35 @@ screen_write_insertmode(struct screen_write_ctx *ctx, int state)
 		s->mode &= ~MODE_INSERT;
 }
 
-/* Set mouse mode.  */
+/* Set UTF-8 mouse mode.  */
 void
-screen_write_mousemode(struct screen_write_ctx *ctx, int state)
+screen_write_utf8mousemode(struct screen_write_ctx *ctx, int state)
 {
 	struct screen	*s = ctx->s;
 
 	if (state)
-		s->mode |= MODE_MOUSE;
+		s->mode |= MODE_MOUSE_UTF8;
 	else
-		s->mode &= ~MODE_MOUSE;
+		s->mode &= ~MODE_MOUSE_UTF8;
+}
+
+/* Set mouse mode off. */
+void
+screen_write_mousemode_off(struct screen_write_ctx *ctx)
+{
+	struct screen	*s = ctx->s;
+
+	s->mode &= ~ALL_MOUSE_MODES;
+}
+
+/* Set mouse mode on. */
+void
+screen_write_mousemode_on(struct screen_write_ctx *ctx, int mode)
+{
+	struct screen	*s = ctx->s;
+
+	s->mode &= ~ALL_MOUSE_MODES;
+	s->mode |= mode;
 }
 
 /* Line feed. */
@@ -912,9 +931,14 @@ screen_write_clearendofscreen(struct screen_write_ctx *ctx)
 	sx = screen_size_x(s);
 	sy = screen_size_y(s);
 
-	if (s->cx <= sx - 1)
-		grid_view_clear(s->grid, s->cx, s->cy, sx - s->cx, 1);
-	grid_view_clear(s->grid, 0, s->cy + 1, sx, sy - (s->cy + 1));
+	/* Scroll into history if it is enabled and clearing entire screen. */
+	if (s->cy == 0 && s->grid->flags & GRID_HISTORY)
+		grid_view_clear_history(s->grid);
+	else {
+		if (s->cx <= sx - 1)
+			grid_view_clear(s->grid, s->cx, s->cy, sx - s->cx, 1);
+		grid_view_clear(s->grid, 0, s->cy + 1, sx, sy - (s->cy + 1));
+	}
 
 	tty_write(tty_cmd_clearendofscreen, &ttyctx);
 }
@@ -950,7 +974,13 @@ screen_write_clearscreen(struct screen_write_ctx *ctx)
 
 	screen_write_initctx(ctx, &ttyctx, 0);
 
-	grid_view_clear(s->grid, 0, 0, screen_size_x(s), screen_size_y(s));
+	/* Scroll into history if it is enabled. */
+	if (s->grid->flags & GRID_HISTORY)
+		grid_view_clear_history(s->grid);
+	else {
+		grid_view_clear(
+		    s->grid, 0, 0, screen_size_x(s), screen_size_y(s));
+	}
 
 	tty_write(tty_cmd_clearscreen, &ttyctx);
 }
@@ -982,8 +1012,10 @@ screen_write_cell(struct screen_write_ctx *ctx,
 	 * If this is a wide character and there is no room on the screen, for
 	 * the entire character, don't print it.
 	 */
-	if (width > 1 && (width > screen_size_x(s) ||
-	    (s->cx != screen_size_x(s) && s->cx > screen_size_x(s) - width)))
+	if (!(s->mode & MODE_WRAP)
+	    && (width > 1 && (width > screen_size_x(s) ||
+		(s->cx != screen_size_x(s)
+		 && s->cx > screen_size_x(s) - width))))
 		return;
 
 	/*
@@ -1015,7 +1047,7 @@ screen_write_cell(struct screen_write_ctx *ctx,
 	}
 
 	/* Sanity checks. */
-	if (((s->mode & MODE_WRAP) && s->cx > screen_size_x(s) - 1)
+	if (((s->mode & MODE_WRAP) && s->cx > screen_size_x(s) - width)
 	    || s->cy > screen_size_y(s) - 1)
 		return;
 
@@ -1161,3 +1193,27 @@ screen_write_overwrite(struct screen_write_ctx *ctx, u_int width)
 		grid_view_set_cell(gd, xx, s->cy, &grid_default_cell);
 	}
 }
+
+void
+screen_write_setselection(struct screen_write_ctx *ctx, u_char *str, u_int len)
+{
+	struct tty_ctx	ttyctx;
+
+	screen_write_initctx(ctx, &ttyctx, 0);
+	ttyctx.ptr = str;
+	ttyctx.num = len;
+
+	tty_write(tty_cmd_setselection, &ttyctx);
+}
+
+void
+screen_write_rawstring(struct screen_write_ctx *ctx, u_char *str, u_int len)
+{
+	struct tty_ctx		 ttyctx;
+
+	screen_write_initctx(ctx, &ttyctx, 0);
+	ttyctx.ptr = str;
+	ttyctx.num = len;
+
+	tty_write(tty_cmd_rawstring, &ttyctx);
+}
