@@ -1,4 +1,4 @@
-/* $Id: server-client.c,v 1.48 2010/12/22 15:31:00 tcunha Exp $ */
+/* $Id: server-client.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2009 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -52,13 +52,9 @@ void
 server_client_create(int fd)
 {
 	struct client	*c;
-	int		 mode;
 	u_int		 i;
 
-	if ((mode = fcntl(fd, F_GETFL)) == -1)
-		fatal("fcntl failed");
-	if (fcntl(fd, F_SETFL, mode|O_NONBLOCK) == -1)
-		fatal("fcntl failed");
+	setblocking(fd, 0);
 
 	c = xcalloc(1, sizeof *c);
 	c->references = 0;
@@ -82,7 +78,8 @@ server_client_create(int fd)
 	c->tty.sy = 24;
 
 	screen_init(&c->status, c->tty.sx, 1, 0);
-	job_tree_init(&c->status_jobs);
+	RB_INIT(&c->status_new);
+	RB_INIT(&c->status_old);
 
 	c->message_string = NULL;
 	ARRAY_INIT(&c->message_log);
@@ -91,6 +88,9 @@ server_client_create(int fd)
 	c->prompt_buffer = NULL;
 	c->prompt_index = 0;
 
+	c->last_mouse.b = MOUSE_UP;
+	c->last_mouse.x = c->last_mouse.y = -1;
+
 	evtimer_set(&c->repeat_timer, server_client_repeat_timer, c);
 
 	for (i = 0; i < ARRAY_LENGTH(&clients); i++) {
@@ -123,21 +123,28 @@ server_client_lost(struct client *c)
 	if (c->flags & CLIENT_TERMINAL)
 		tty_free(&c->tty);
 
-	if (c->stdin_fd != -1)
+	if (c->stdin_fd != -1) {
+		setblocking(c->stdin_fd, 1);
 		close(c->stdin_fd);
+	}
 	if (c->stdin_event != NULL)
 		bufferevent_free(c->stdin_event);
-	if (c->stdout_fd != -1)
+	if (c->stdout_fd != -1) {
+		setblocking(c->stdout_fd, 1);
 		close(c->stdout_fd);
+	}
 	if (c->stdout_event != NULL)
 		bufferevent_free(c->stdout_event);
-	if (c->stderr_fd != -1)
+	if (c->stderr_fd != -1) {
+		setblocking(c->stderr_fd, 1);
 		close(c->stderr_fd);
+	}
 	if (c->stderr_event != NULL)
 		bufferevent_free(c->stderr_event);
 
+	status_free_jobs(&c->status_new);
+	status_free_jobs(&c->status_old);
 	screen_free(&c->status);
-	job_tree_free(&c->status_jobs);
 
 	if (c->title != NULL)
 		xfree(c->title);
@@ -218,7 +225,6 @@ server_client_status_timer(void)
 {
 	struct client	*c;
 	struct session	*s;
-	struct job	*job;
 	struct timeval	 tv;
 	u_int		 i;
 	int		 interval;
@@ -247,8 +253,7 @@ server_client_status_timer(void)
 
 		difference = tv.tv_sec - c->status_timer.tv_sec;
 		if (difference >= interval) {
-			RB_FOREACH(job, jobs, &c->status_jobs)
-				job_run(job);
+			status_update_jobs(c);
 			c->flags |= CLIENT_STATUS;
 		}
 	}
@@ -311,11 +316,40 @@ server_client_handle_key(int key, struct mouse_event *mouse, void *data)
 	if (key == KEYC_MOUSE) {
 		if (c->flags & CLIENT_READONLY)
 			return;
-		if (options_get_number(oo, "mouse-select-pane")) {
+		if (options_get_number(oo, "mouse-select-pane") &&
+		    ((!(mouse->b & MOUSE_DRAG) && mouse->b != MOUSE_UP) ||
+		    wp->mode != &window_copy_mode)) {
+			/*
+			 * Allow pane switching in copy mode only by mouse down
+			 * (click).
+			 */
 			window_set_active_at(w, mouse->x, mouse->y);
 			server_redraw_window_borders(w);
 			wp = w->active;
 		}
+		if (mouse->y + 1 == c->tty.sy &&
+		    options_get_number(oo, "mouse-select-window") &&
+		    options_get_number(oo, "status")) {
+			if (mouse->b == MOUSE_UP &&
+			    c->last_mouse.b != MOUSE_UP) {
+				status_set_window_at(c, mouse->x);
+				return;
+			}
+			if (mouse->b & MOUSE_45) {
+				if ((mouse->b & MOUSE_BUTTON) == MOUSE_1) {
+					session_previous(c->session, 0);
+					server_redraw_session(s);
+				}
+				if ((mouse->b & MOUSE_BUTTON) == MOUSE_2) {
+					session_next(c->session, 0);
+					server_redraw_session(s);
+				}
+				return;
+			}
+		}
+		if (options_get_number(oo, "mouse-resize-pane"))
+			layout_resize_pane_mouse(c, mouse);
+		memcpy(&c->last_mouse, mouse, sizeof c->last_mouse);
 		window_pane_mouse(wp, c->session, mouse);
 		return;
 	}
@@ -436,8 +470,12 @@ server_client_reset_state(struct client *c)
 	struct window_pane	*wp = w->active;
 	struct screen		*s = wp->screen;
 	struct options		*oo = &c->session->options;
+	struct options		*wo = &w->options;
 	int			 status, mode;
 
+	if (c->flags & CLIENT_SUSPENDED)
+		return;
+
 	tty_region(&c->tty, 0, c->tty.sy - 1);
 
 	status = options_get_number(oo, "status");
@@ -446,11 +484,46 @@ server_client_reset_state(struct client *c)
 	else
 		tty_cursor(&c->tty, wp->xoff + s->cx, wp->yoff + s->cy);
 
+	/*
+	 * Resizing panes with the mouse requires at least button mode to give
+	 * a smooth appearance.
+	 */
 	mode = s->mode;
-	if (TAILQ_NEXT(TAILQ_FIRST(&w->panes), entry) != NULL &&
-	    options_get_number(oo, "mouse-select-pane"))
-		mode |= MODE_MOUSE;
-	tty_update_mode(&c->tty, mode);
+	if ((c->last_mouse.b & MOUSE_RESIZE_PANE) &&
+	    !(mode & (MODE_MOUSE_BUTTON|MODE_MOUSE_ANY)))
+		mode |= MODE_MOUSE_BUTTON;
+
+	/*
+	 * Any mode will do for mouse-select-pane, but set standard mode if
+	 * none.
+	 */
+	if ((mode & ALL_MOUSE_MODES) == 0) {
+		if (TAILQ_NEXT(TAILQ_FIRST(&w->panes), entry) != NULL &&
+		    options_get_number(oo, "mouse-select-pane"))
+			mode |= MODE_MOUSE_STANDARD;
+		else if (options_get_number(oo, "mouse-resize-pane"))
+			mode |= MODE_MOUSE_STANDARD;
+		else if (options_get_number(oo, "mouse-select-window"))
+			mode |= MODE_MOUSE_STANDARD;
+		else if (options_get_number(wo, "mode-mouse"))
+			mode |= MODE_MOUSE_STANDARD;
+	}
+
+	/*
+	 * Set UTF-8 mouse input if required. If the terminal is UTF-8, the
+	 * user has set mouse-utf8 and any mouse mode is in effect, turn on
+	 * UTF-8 mouse input. If the receiving terminal hasn't requested it
+	 * (that is, it isn't in s->mode), then it'll be converted in
+	 * input_mouse.
+	 */
+	if ((c->tty.flags & TTY_UTF8) &&
+	    (mode & ALL_MOUSE_MODES) && options_get_number(oo, "mouse-utf8"))
+		mode |= MODE_MOUSE_UTF8;
+	else
+		mode &= ~MODE_MOUSE_UTF8;
+
+	/* Set the terminal mode and reset attributes. */
+	tty_update_mode(&c->tty, mode, s);
 	tty_reset(&c->tty);
 }
 
@@ -591,7 +664,7 @@ server_client_set_title(struct client *c)
 
 	template = options_get_string(&s->options, "set-titles-string");
 
-	title = status_replace(c, NULL, template, time(NULL), 1);
+	title = status_replace(c, NULL, NULL, NULL, template, time(NULL), 1);
 	if (c->title == NULL || strcmp(title, c->title) != 0) {
 		if (c->title != NULL)
 			xfree(c->title);
@@ -616,6 +689,7 @@ server_client_in_callback(
 		return;
 
 	bufferevent_disable(c->stdin_event, EV_READ|EV_WRITE);
+	setblocking(c->stdin_fd, 1);
 	close(c->stdin_fd);
 	c->stdin_fd = -1;
 
@@ -631,6 +705,7 @@ server_client_out_callback(
 	struct client	*c = data;
 
 	bufferevent_disable(c->stdout_event, EV_READ|EV_WRITE);
+	setblocking(c->stdout_fd, 1);
 	close(c->stdout_fd);
 	c->stdout_fd = -1;
 }
@@ -643,6 +718,7 @@ server_client_err_callback(
 	struct client	*c = data;
 
 	bufferevent_disable(c->stderr_event, EV_READ|EV_WRITE);
+	setblocking(c->stderr_fd, 1);
 	close(c->stderr_fd);
 	c->stderr_fd = -1;
 }
@@ -656,7 +732,6 @@ server_client_msg_dispatch(struct client *c)
 	struct msg_identify_data identifydata;
 	struct msg_environ_data	 environdata;
 	ssize_t			 n, datalen;
-	int			 mode;
 
 	if ((n = imsg_read(&c->ibuf)) == -1 || n == 0)
 		return (-1);
@@ -696,9 +771,7 @@ server_client_msg_dispatch(struct client *c)
 			    NULL, NULL, server_client_in_callback, c);
 			if (c->stdin_event == NULL)
 				fatalx("failed to create stdin event");
-
-			if ((mode = fcntl(c->stdin_fd, F_GETFL)) != -1)
-				fcntl(c->stdin_fd, F_SETFL, mode|O_NONBLOCK);
+			setblocking(c->stdin_fd, 0);
 
 			server_client_msg_identify(c, &identifydata, imsg.fd);
 			break;
@@ -713,9 +786,8 @@ server_client_msg_dispatch(struct client *c)
 			    NULL, NULL, server_client_out_callback, c);
 			if (c->stdout_event == NULL)
 				fatalx("failed to create stdout event");
+			setblocking(c->stdout_fd, 0);
 
-			if ((mode = fcntl(c->stdout_fd, F_GETFL)) != -1)
-				fcntl(c->stdout_fd, F_SETFL, mode|O_NONBLOCK);
 			break;
 		case MSG_STDERR:
 			if (datalen != 0)
@@ -728,9 +800,8 @@ server_client_msg_dispatch(struct client *c)
 			    NULL, NULL, server_client_err_callback, c);
 			if (c->stderr_event == NULL)
 				fatalx("failed to create stderr event");
+			setblocking(c->stderr_fd, 0);
 
-			if ((mode = fcntl(c->stderr_fd, F_GETFL)) != -1)
-				fcntl(c->stderr_fd, F_SETFL, mode|O_NONBLOCK);
 			break;
 		case MSG_RESIZE:
 			if (datalen != 0)
@@ -760,11 +831,8 @@ server_client_msg_dispatch(struct client *c)
 
 			if (gettimeofday(&c->activity_time, NULL) != 0)
 				fatal("gettimeofday");
-			if (c->session != NULL) {
-				memcpy(&c->session->activity_time,
-				    &c->activity_time,
-				    sizeof c->session->activity_time);
-			}
+			if (c->session != NULL)
+				session_update_activity(c->session);
 
 			tty_start_tty(&c->tty);
 			server_redraw_client(c);
