@@ -1,4 +1,4 @@
-/* $Id: session.c,v 1.83 2010/12/22 15:36:44 tcunha Exp $ */
+/* $Id: session.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -96,7 +96,7 @@ session_create(const char *name, const char *cmd, const char *cwd,
 
 	if (gettimeofday(&s->creation_time, NULL) != 0)
 		fatal("gettimeofday failed");
-	memcpy(&s->activity_time, &s->creation_time, sizeof s->activity_time);
+	session_update_activity(s);
 
 	s->cwd = xstrdup(cwd);
 
@@ -104,8 +104,6 @@ session_create(const char *name, const char *cmd, const char *cwd,
 	TAILQ_INIT(&s->lastw);
 	RB_INIT(&s->windows);
 
-	paste_init_stack(&s->buffers);
-
 	options_init(&s->options, &global_s_options);
 	environ_init(&s->environ);
 	if (env != NULL)
@@ -154,7 +152,6 @@ session_destroy(struct session *s)
 	session_group_remove(s);
 	environ_free(&s->environ);
 	options_free(&s->options);
-	paste_free_stack(&s->buffers);
 
 	while (!TAILQ_EMPTY(&s->lastw))
 		winlink_stack_remove(&s->lastw, TAILQ_FIRST(&s->lastw));
@@ -166,6 +163,21 @@ session_destroy(struct session *s)
 	RB_INSERT(sessions, &dead_sessions, s);
 }
 
+/* Check a session name is valid: not empty and no colons. */
+int
+session_check_name(const char *name)
+{
+	return (*name != '\0' && strchr(name, ':') == NULL);
+}
+
+/* Update session active time. */
+void
+session_update_activity(struct session *s)
+{
+	if (gettimeofday(&s->activity_time, NULL) != 0)
+		fatal("gettimeofday");
+}
+
 /* Find the next usable session. */
 struct session *
 session_next_session(struct session *s)
@@ -175,12 +187,9 @@ session_next_session(struct session *s)
 	if (RB_EMPTY(&sessions) || !session_alive(s))
 		return (NULL);
 
-	s2 = s;
-	do {
-		s2 = RB_NEXT(sessions, &sessions, s2);
-		if (s2 == NULL)
-			s2 = RB_MIN(sessions, &sessions);
-	} while (s2 != s);
+	s2 = RB_NEXT(sessions, &sessions, s);
+	if (s2 == NULL)
+		s2 = RB_MIN(sessions, &sessions);
 	if (s2 == s)
 		return (NULL);
 	return (s2);
@@ -195,12 +204,9 @@ session_previous_session(struct session *s)
 	if (RB_EMPTY(&sessions) || !session_alive(s))
 		return (NULL);
 
-	s2 = s;
-	do {
-		s2 = RB_PREV(sessions, &sessions, s2);
-		if (s2 == NULL)
-			s2 = RB_MAX(sessions, &sessions);
-	} while (s2 != s);
+	s2 = RB_PREV(sessions, &sessions, s);
+	if (s2 == NULL)
+		s2 = RB_MAX(sessions, &sessions);
 	if (s2 == s)
 		return (NULL);
 	return (s2);
@@ -212,10 +218,16 @@ session_new(struct session *s,
     const char *name, const char *cmd, const char *cwd, int idx, char **cause)
 {
 	struct window	*w;
+	struct winlink	*wl;
 	struct environ	 env;
 	const char	*shell;
 	u_int		 hlimit;
 
+	if ((wl = winlink_add(&s->windows, idx)) == NULL) {
+		xasprintf(cause, "index in use: %d", idx);
+		return (NULL);
+	}
+
 	environ_init(&env);
 	environ_copy(&global_environ, &env);
 	environ_copy(&s->environ, &env);
@@ -229,15 +241,18 @@ session_new(struct session *s,
 	w = window_create(
 	    name, cmd, shell, cwd, &env, s->tio, s->sx, s->sy, hlimit, cause);
 	if (w == NULL) {
+		winlink_remove(&s->windows, wl);
 		environ_free(&env);
 		return (NULL);
 	}
+	winlink_set_window(wl, w);
 	environ_free(&env);
 
 	if (options_get_number(&s->options, "set-remain-on-exit"))
 		options_set_number(&w->options, "remain-on-exit", 1);
 
-	return (session_attach(s, w, idx, cause));
+	session_group_synchronize_from(s);
+	return (wl);
 }
 
 /* Attach a window to a session. */
@@ -246,8 +261,12 @@ session_attach(struct session *s, struct window *w, int idx, char **cause)
 {
 	struct winlink	*wl;
 
-	if ((wl = winlink_add(&s->windows, w, idx)) == NULL)
+	if ((wl = winlink_add(&s->windows, idx)) == NULL) {
 		xasprintf(cause, "index in use: %d", idx);
+		return (NULL);
+	}
+	winlink_set_window(wl, w);
+
 	session_group_synchronize_from(s);
 	return (wl);
 }
@@ -526,7 +545,8 @@ session_group_synchronize1(struct session *target, struct session *s)
 
 	/* Link all the windows from the target. */
 	RB_FOREACH(wl, winlinks, ww) {
-		wl2 = winlink_add(&s->windows, wl->window, wl->idx);
+		wl2 = winlink_add(&s->windows, wl->idx);
+		winlink_set_window(wl2, wl->window);
 		wl2->flags |= wl->flags & WINLINK_ALERTFLAGS;
 	}
 
