@@ -1,4 +1,4 @@
-/* $Id: tmux.c,v 1.228 2010/12/27 21:22:24 tcunha Exp $ */
+/* $Id: tmux.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -47,14 +47,14 @@ time_t		 start_time;
 char		 socket_path[MAXPATHLEN];
 int		 login_shell;
 char		*environ_path;
-pid_t		 environ_pid;
-u_int		 environ_idx;
+pid_t		 environ_pid = -1;
+int		 environ_idx = -1;
 
 __dead void	 usage(void);
 void	 	 parseenvironment(void);
 char 		*makesocketpath(const char *);
 
-#ifndef HAVE_PROGNAME
+#ifndef HAVE___PROGNAME
 char      *__progname = (char *) "tmux";
 #endif
 
@@ -128,56 +128,32 @@ areshell(const char *shell)
 void
 parseenvironment(void)
 {
-	char		*env, *path_pid, *pid_idx, buf[256];
-	size_t		 len;
-	const char	*errstr;
-	long long	 ll;
+	char	*env, path[256];
+	long	 pid;
+	int	 idx;
 
-	environ_pid = -1;
 	if ((env = getenv("TMUX")) == NULL)
 		return;
 
-	if ((path_pid = strchr(env, ',')) == NULL || path_pid == env)
+	if (sscanf(env, "%255[^,],%ld,%d", path, &pid, &idx) != 3)
 		return;
-	if ((pid_idx = strchr(path_pid + 1, ',')) == NULL)
-		return;
-	if ((pid_idx == path_pid + 1 || pid_idx[1] == '\0'))
-		return;
-
-	/* path */
-	len = path_pid - env;
-	environ_path = xmalloc(len + 1);
-	memcpy(environ_path, env, len);
-	environ_path[len] = '\0';
-
-	/* pid */
-	len = pid_idx - path_pid - 1;
-	if (len > (sizeof buf) - 1)
-		return;
-	memcpy(buf, path_pid + 1, len);
-	buf[len] = '\0';
-
-	ll = strtonum(buf, 0, LONG_MAX, &errstr);
-	if (errstr != NULL)
-		return;
-	environ_pid = ll;
-
-	/* idx */
-	ll = strtonum(pid_idx + 1, 0, UINT_MAX, &errstr);
-	if (errstr != NULL)
-		return;
-	environ_idx = ll;
+	environ_path = xstrdup(path);
+	environ_pid = pid;
+	environ_idx = idx;
 }
 
 char *
 makesocketpath(const char *label)
 {
-	char		base[MAXPATHLEN], *path;
+	char		base[MAXPATHLEN], *path, *s;
 	struct stat	sb;
 	u_int		uid;
 
 	uid = getuid();
-	xsnprintf(base, MAXPATHLEN, "%s/tmux-%d", _PATH_TMP, uid);
+	if ((s = getenv("TMPDIR")) == NULL || *s == '\0')
+		xsnprintf(base, sizeof base, "%s/tmux-%u", _PATH_TMP, uid);
+	else
+		xsnprintf(base, sizeof base, "%s/tmux-%u", s, uid);
 
 	if (mkdir(base, S_IRWXU) != 0 && errno != EEXIST)
 		return (NULL);
@@ -197,12 +173,25 @@ makesocketpath(const char *label)
 	return (path);
 }
 
+void
+setblocking(int fd, int state)
+{
+	int mode;
+
+	if ((mode = fcntl(fd, F_GETFL)) != -1) {
+		if (!state)
+			mode |= O_NONBLOCK;
+		else
+			mode &= ~O_NONBLOCK;
+		fcntl(fd, F_SETFL, mode);
+	}
+}
+
 __dead void
 shell_exec(const char *shell, const char *shellcmd)
 {
 	const char	*shellname, *ptr;
 	char		*argv0;
-	int		 mode;
 
 	ptr = strrchr(shell, '/');
 	if (ptr != NULL && *(ptr + 1) != '\0')
@@ -215,12 +204,9 @@ shell_exec(const char *shell, const char *shellcmd)
 		xasprintf(&argv0, "%s", shellname);
 	setenv("SHELL", shell, 1);
 
-	if ((mode = fcntl(STDIN_FILENO, F_GETFL)) != -1)
-		fcntl(STDIN_FILENO, F_SETFL, mode & ~O_NONBLOCK);
-	if ((mode = fcntl(STDOUT_FILENO, F_GETFL)) != -1)
-		fcntl(STDOUT_FILENO, F_SETFL, mode & ~O_NONBLOCK);
-	if ((mode = fcntl(STDERR_FILENO, F_GETFL)) != -1)
-		fcntl(STDERR_FILENO, F_SETFL, mode & ~O_NONBLOCK);
+	setblocking(STDIN_FILENO, 1);
+	setblocking(STDOUT_FILENO, 1);
+	setblocking(STDERR_FILENO, 1);
 	closefrom(STDERR_FILENO + 1);
 
 	execl(shell, argv0, "-c", shellcmd, (char *) NULL);
@@ -231,7 +217,6 @@ int
 main(int argc, char **argv)
 {
 	struct passwd	*pw;
-	struct options	*oo, *so, *wo;
 	struct keylist	*keylist;
 	char		*s, *path, *label, *home, **var;
 	int	 	 opt, flags, quiet, keys;
@@ -259,7 +244,7 @@ main(int argc, char **argv)
 			shell_cmd = xstrdup(optarg);
 			break;
 		case 'V':
-			printf("%s %s\n", __progname, BUILD);
+			printf("%s %s\n", __progname, VERSION);
 			exit(0);
 		case 'f':
 			if (cfg_file != NULL)
@@ -322,129 +307,40 @@ main(int argc, char **argv)
 		environ_put(&global_environ, *var);
 
 	options_init(&global_options, NULL);
-	oo = &global_options;
-	options_set_number(oo, "quiet", quiet);
-	options_set_number(oo, "escape-time", 500);
-	options_set_number(oo, "exit-unattached", 0);
+	options_table_populate_tree(server_options_table, &global_options);
+	options_set_number(&global_options, "quiet", quiet);
 
 	options_init(&global_s_options, NULL);
-	so = &global_s_options;
-	options_set_number(so, "base-index", 0);
-	options_set_number(so, "bell-action", BELL_ANY);
-	options_set_number(so, "buffer-limit", 9);
-	options_set_string(so, "default-command", "%s", "");
-	options_set_string(so, "default-path", "%s", "");
-	options_set_string(so, "default-shell", "%s", getshell());
-	options_set_string(so, "default-terminal", "screen");
-	options_set_number(so, "destroy-unattached", 0);
-	options_set_number(so, "detach-on-destroy", 1);
-	options_set_number(so, "display-panes-active-colour", 1);
-	options_set_number(so, "display-panes-colour", 4);
-	options_set_number(so, "display-panes-time", 1000);
-	options_set_number(so, "display-time", 750);
-	options_set_number(so, "history-limit", 2000);
-	options_set_number(so, "lock-after-time", 0);
-	options_set_string(so, "lock-command", "lock -np");
-	options_set_number(so, "lock-server", 1);
-	options_set_number(so, "message-attr", 0);
-	options_set_number(so, "message-bg", 3);
-	options_set_number(so, "message-fg", 0);
-	options_set_number(so, "message-limit", 20);
-	options_set_number(so, "mouse-select-pane", 0);
-	options_set_number(so, "pane-active-border-bg", 8);
-	options_set_number(so, "pane-active-border-fg", 2);
-	options_set_number(so, "pane-border-bg", 8);
-	options_set_number(so, "pane-border-fg", 8);
-	options_set_number(so, "repeat-time", 500);
-	options_set_number(so, "set-remain-on-exit", 0);
-	options_set_number(so, "set-titles", 0);
-	options_set_string(so, "set-titles-string", "#S:#I:#W - \"#T\"");
-	options_set_number(so, "status", 1);
-	options_set_number(so, "status-attr", 0);
-	options_set_number(so, "status-bg", 2);
-	options_set_number(so, "status-fg", 0);
-	options_set_number(so, "status-interval", 15);
-	options_set_number(so, "status-justify", 0);
-	options_set_string(so, "status-left", "[#S]");
-	options_set_number(so, "status-left-attr", 0);
-	options_set_number(so, "status-left-bg", 8);
-	options_set_number(so, "status-left-fg", 8);
-	options_set_number(so, "status-left-length", 10);
-	options_set_string(so, "status-right", "\"#22T\" %%H:%%M %%d-%%b-%%y");
-	options_set_number(so, "status-right-attr", 0);
-	options_set_number(so, "status-right-bg", 8);
-	options_set_number(so, "status-right-fg", 8);
-	options_set_number(so, "status-right-length", 40);
-	options_set_string(so, "terminal-overrides",
-	    "*88col*:colors=88,*256col*:colors=256");
-	options_set_string(so, "update-environment",
-	    "DISPLAY "
-	    "SSH_ASKPASS SSH_AUTH_SOCK SSH_AGENT_PID SSH_CONNECTION "
-	    "WINDOWID "
-	    "XAUTHORITY");
-	options_set_number(so, "visual-activity", 0);
-	options_set_number(so, "visual-bell", 0);
-	options_set_number(so, "visual-content", 0);
-	options_set_number(so, "visual-silence", 0);
+	options_table_populate_tree(session_options_table, &global_s_options);
+	options_set_string(&global_s_options, "default-shell", "%s", getshell());
 
+	options_init(&global_w_options, NULL);
+	options_table_populate_tree(window_options_table, &global_w_options);
+
+	/* Set the prefix option (its a list, so not in the table). */
 	keylist = xmalloc(sizeof *keylist);
 	ARRAY_INIT(keylist);
 	ARRAY_ADD(keylist, '\002');
-	options_set_data(so, "prefix", keylist, xfree);
-
-	options_init(&global_w_options, NULL);
-	wo = &global_w_options;
-	options_set_number(wo, "aggressive-resize", 0);
-	options_set_number(wo, "alternate-screen", 1);
-	options_set_number(wo, "automatic-rename", 1);
-	options_set_number(wo, "clock-mode-colour", 4);
-	options_set_number(wo, "clock-mode-style", 1);
-	options_set_number(wo, "force-height", 0);
-	options_set_number(wo, "force-width", 0);
-	options_set_number(wo, "main-pane-height", 24);
-	options_set_number(wo, "main-pane-width", 80);
-	options_set_number(wo, "mode-attr", 0);
-	options_set_number(wo, "mode-bg", 3);
-	options_set_number(wo, "mode-fg", 0);
-	options_set_number(wo, "mode-mouse", 0);
-	options_set_number(wo, "monitor-activity", 0);
-	options_set_string(wo, "monitor-content", "%s", "");
-	options_set_number(wo, "monitor-silence", 0);
-	options_set_number(wo, "other-pane-height", 0);
-	options_set_number(wo, "other-pane-width", 0);
-	options_set_number(wo, "window-status-attr", 0);
-	options_set_number(wo, "window-status-bg", 8);
-	options_set_number(wo, "window-status-current-attr", 0);
-	options_set_number(wo, "window-status-current-bg", 8);
-	options_set_number(wo, "window-status-current-fg", 8);
-	options_set_number(wo, "window-status-fg", 8);
-	options_set_number(wo, "window-status-alert-attr", GRID_ATTR_REVERSE);
-	options_set_number(wo, "window-status-alert-bg", 8);
-	options_set_number(wo, "window-status-alert-fg", 8);
-	options_set_string(wo, "window-status-format", "#I:#W#F");
-	options_set_string(wo, "window-status-current-format", "#I:#W#F");
-	options_set_string(wo, "word-separators", " -_@");
-	options_set_number(wo, "xterm-keys", 0);
-	options_set_number(wo, "remain-on-exit", 0);
-	options_set_number(wo, "synchronize-panes", 0);
+	options_set_data(&global_s_options, "prefix", keylist, xfree);
 
+	/* Enable UTF-8 if the first client is on UTF-8 terminal. */
 	if (flags & IDENTIFY_UTF8) {
-		options_set_number(so, "status-utf8", 1);
-		options_set_number(wo, "utf8", 1);
-	} else {
-		options_set_number(so, "status-utf8", 0);
-		options_set_number(wo, "utf8", 0);
+		options_set_number(&global_s_options, "status-utf8", 1);
+		options_set_number(&global_s_options, "mouse-utf8", 1);
+		options_set_number(&global_w_options, "utf8", 1);
 	}
 
-	keys = MODEKEY_EMACS;
+	/* Override keys to vi if VISUAL or EDITOR are set. */
 	if ((s = getenv("VISUAL")) != NULL || (s = getenv("EDITOR")) != NULL) {
 		if (strrchr(s, '/') != NULL)
 			s = strrchr(s, '/') + 1;
 		if (strstr(s, "vi") != NULL)
 			keys = MODEKEY_VI;
+		else
+			keys = MODEKEY_EMACS;
+		options_set_number(&global_s_options, "status-keys", keys);
+		options_set_number(&global_w_options, "mode-keys", keys);
 	}
-	options_set_number(so, "status-keys", keys);
-	options_set_number(wo, "mode-keys", keys);
 
 	/* Locate the configuration file. */
 	if (cfg_file == NULL) {
@@ -495,20 +391,6 @@ main(int argc, char **argv)
 #endif
 
 	/* Pass control to the client. */
-#ifdef HAVE_BROKEN_KQUEUE
-	if (setenv("EVENT_NOKQUEUE", "1", 1) != 0)
-		fatal("setenv failed");
-#endif
-#ifdef HAVE_BROKEN_POLL
-	if (setenv("EVENT_NOPOLL", "1", 1) != 0)
-		fatal("setenv failed");
-#endif
-	ev_base = event_init();
-#ifdef HAVE_BROKEN_KQUEUE
-	unsetenv("EVENT_NOKQUEUE");
-#endif
-#ifdef HAVE_BROKEN_POLL
-	unsetenv("EVENT_NOPOLL");
-#endif
+	ev_base = osdep_event_init();
 	exit(client_main(argc, argv, flags));
 }
