@@ -1,4 +1,4 @@
-/* $Id: tty-keys.c,v 1.57 2010/06/06 00:23:44 tcunha Exp $ */
+/* $Id: tty-keys.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -38,7 +38,7 @@ struct tty_key *tty_keys_find1(
 		    struct tty_key *, const char *, size_t, size_t *);
 struct tty_key *tty_keys_find(struct tty *, const char *, size_t, size_t *);
 void		tty_keys_callback(int, short, void *);
-int		tty_keys_mouse(
+int		tty_keys_mouse(struct tty *,
 		    const char *, size_t, size_t *, struct mouse_event *);
 
 struct tty_key_ent {
@@ -54,7 +54,7 @@ struct tty_key_ent {
  * Default key tables. Those flagged with TTYKEY_RAW are inserted directly,
  * otherwise they are looked up in terminfo(5).
  */
-struct tty_key_ent tty_keys[] = {
+const struct tty_key_ent tty_keys[] = {
 	/*
 	 * Numeric keypad. Just use the vt100 escape sequences here and always
 	 * put the terminal into keypad_xmit mode. Translation of numbers
@@ -343,9 +343,9 @@ tty_keys_add1(struct tty_key **tkp, const char *s, int key)
 void
 tty_keys_init(struct tty *tty)
 {
-	struct tty_key_ent	*tke;
-	u_int		 	 i;
-	const char		*s;
+	const struct tty_key_ent	*tke;
+	u_int		 		 i;
+	const char			*s;
 
 	tty->key_tree = NULL;
 	for (i = 0; i < nitems(tty_keys); i++) {
@@ -462,7 +462,7 @@ tty_keys_next(struct tty *tty)
 	}
 
 	/* Is this a mouse key press? */
-	switch (tty_keys_mouse(buf, len, &size, &mouse)) {
+	switch (tty_keys_mouse(tty, buf, len, &size, &mouse)) {
 	case 0:		/* yes */
 		evbuffer_drain(tty->event->input, size);
 		key = KEYC_MOUSE;
@@ -584,44 +584,74 @@ tty_keys_callback(unused int fd, unused short events, void *data)
  * (probably a mouse sequence but need more data).
  */
 int
-tty_keys_mouse(const char *buf, size_t len, size_t *size, struct mouse_event *m)
+tty_keys_mouse(struct tty *tty,
+    const char *buf, size_t len, size_t *size, struct mouse_event *m)
 {
+	struct utf8_data	utf8data;
+	u_int			i, value;
+
 	/*
-	 * Mouse sequences are \033[M followed by three characters indicating
-	 * buttons, X and Y, all based at 32 with 1,1 top-left.
+	 * Standard mouse sequences are \033[M followed by three characters
+	 * indicating buttons, X and Y, all based at 32 with 1,1 top-left.
+	 *
+	 * UTF-8 mouse sequences are similar but the three are expressed as
+	 * UTF-8 characters.
 	 */
 
 	*size = 0;
 
+	/* First three bytes are always \033[M. */
 	if (buf[0] != '\033')
 		return (-1);
 	if (len == 1)
 		return (1);
-
 	if (buf[1] != '[')
 		return (-1);
 	if (len == 2)
 		return (1);
-
 	if (buf[2] != 'M')
 		return (-1);
 	if (len == 3)
 		return (1);
 
-	if (len < 6)
-		return (1);
-	*size = 6;
+	/* Read the three inputs. */
+	*size = 3;
+	for (i = 0; i < 3; i++) {
+		if (len < *size)
+			return (1);
 
-	log_debug(
-	    "mouse input: %.6s (%hhu,%hhu/%hhu)", buf, buf[4], buf[5], buf[3]);
+		if (tty->mode & MODE_MOUSE_UTF8) {
+			if (utf8_open(&utf8data, buf[*size])) {
+				if (utf8data.size != 2)
+					return (-1);
+				(*size)++;
+				if (len < *size)
+					return (1);
+				utf8_append(&utf8data, buf[*size]);
+				value = utf8_combine(&utf8data);
+			} else
+				value = (unsigned char)buf[*size];
+			(*size)++;
+		} else {
+			value = (unsigned char)buf[*size];
+			(*size)++;
+		}
 
-	m->b = buf[3];
-	m->x = buf[4];
-	m->y = buf[5];
+		if (i == 0)
+			m->b = value;
+		else if (i == 1)
+			m->x = value;
+		else
+			m->y = value;
+	}
+	log_debug("mouse input: %.*s", (int) *size, buf);
+
+	/* Check and return the mouse input. */
 	if (m->b < 32 || m->x < 33 || m->y < 33)
 		return (-1);
 	m->b -= 32;
 	m->x -= 33;
 	m->y -= 33;
+	log_debug("mouse position: x=%u y=%u b=%u", m->x, m->y, m->b);
 	return (0);
 }
