@@ -1,4 +1,4 @@
-/* $Id: window.c,v 1.142 2010/12/06 22:52:21 nicm Exp $ */
+/* $Id: window.c 2553 2011-07-09 09:42:33Z tcunha $ */
 
 /*
  * Copyright (c) 2007 Nicholas Marriott <nicm@users.sourceforge.net>
@@ -53,6 +53,10 @@
 /* Global window list. */
 struct windows windows;
 
+/* Global panes tree. */
+struct window_pane_tree all_window_panes;
+u_int	next_window_pane;
+
 void	window_pane_read_callback(struct bufferevent *, void *);
 void	window_pane_error_callback(struct bufferevent *, short, void *);
 
@@ -64,6 +68,14 @@ winlink_cmp(struct winlink *wl1, struct winlink *wl2)
 	return (wl1->idx - wl2->idx);
 }
 
+RB_GENERATE(window_pane_tree, window_pane, tree_entry, window_pane_cmp);
+
+int
+window_pane_cmp(struct window_pane *wp1, struct window_pane *wp2)
+{
+	return (wp1->id - wp2->id);
+}
+
 struct winlink *
 winlink_find_by_window(struct winlinks *wwl, struct window *w)
 {
@@ -120,7 +132,7 @@ winlink_count(struct winlinks *wwl)
 }
 
 struct winlink *
-winlink_add(struct winlinks *wwl, struct window *w, int idx)
+winlink_add(struct winlinks *wwl, int idx)
 {
 	struct winlink	*wl;
 
@@ -132,14 +144,18 @@ winlink_add(struct winlinks *wwl, struct window *w, int idx)
 
 	wl = xcalloc(1, sizeof *wl);
 	wl->idx = idx;
-	wl->window = w;
 	RB_INSERT(winlinks, wwl, wl);
 
-	w->references++;
-
 	return (wl);
 }
 
+void
+winlink_set_window(struct winlink *wl, struct window *w)
+{
+	wl->window = w;
+	w->references++;
+}
+
 void
 winlink_remove(struct winlinks *wwl, struct winlink *wl)
 {
@@ -150,11 +166,13 @@ winlink_remove(struct winlinks *wwl, struct winlink *wl)
 		xfree(wl->status_text);
 	xfree(wl);
 
-	if (w->references == 0)
-		fatal("bad reference count");
-	w->references--;
-	if (w->references == 0)
-		window_destroy(w);
+	if (w != NULL) {
+		if (w->references == 0)
+			fatal("bad reference count");
+		w->references--;
+		if (w->references == 0)
+			window_destroy(w);
+	}
 }
 
 struct winlink *
@@ -335,21 +353,65 @@ window_set_active_pane(struct window *w, struct window_pane *wp)
 	}
 }
 
+struct window_pane *
+window_get_active_at(struct window *w, u_int x, u_int y)
+{
+	struct window_pane	*wp;
+
+	TAILQ_FOREACH(wp, &w->panes, entry) {
+		if (!window_pane_visible(wp))
+			continue;
+		if (x < wp->xoff || x > wp->xoff + wp->sx)
+			continue;
+		if (y < wp->yoff || y > wp->yoff + wp->sy)
+			continue;
+		return (wp);
+	}
+	return (NULL);
+}
+
 void
 window_set_active_at(struct window *w, u_int x, u_int y)
 {
 	struct window_pane	*wp;
 
-	TAILQ_FOREACH(wp, &w->panes, entry) {
-		if (wp == w->active || !window_pane_visible(wp))
-			continue;
-		if (x < wp->xoff || x >= wp->xoff + wp->sx)
-			continue;
-		if (y < wp->yoff || y >= wp->yoff + wp->sy)
-			continue;
+	wp = window_get_active_at(w, x, y);
+	if (wp != NULL && wp != w->active)
 		window_set_active_pane(w, wp);
-		break;
-	}
+}
+
+struct window_pane *
+window_find_string(struct window *w, const char *s)
+{
+	u_int	x, y;
+
+	x = w->sx / 2;
+	y = w->sy / 2;
+
+	if (strcasecmp(s, "top") == 0)
+		y = 0;
+	else if (strcasecmp(s, "bottom") == 0)
+		y = w->sy - 1;
+	else if (strcasecmp(s, "left") == 0)
+		x = 0;
+	else if (strcasecmp(s, "right") == 0)
+		x = w->sx - 1;
+	else if (strcasecmp(s, "top-left") == 0) {
+		x = 0;
+		y = 0;
+	} else if (strcasecmp(s, "top-right") == 0) {
+		x = w->sx - 1;
+		y = 0;
+	} else if (strcasecmp(s, "bottom-left") == 0) {
+		x = 0;
+		y = w->sy - 1;
+	} else if (strcasecmp(s, "bottom-right") == 0) {
+		x = w->sx - 1;
+		y = w->sy - 1;
+	} else
+		return (NULL);
+
+	return (window_get_active_at(w, x, y));
 }
 
 struct window_pane *
@@ -460,6 +522,42 @@ window_destroy_panes(struct window *w)
 	}
 }
 
+/* Return list of printable window flag symbols. No flags is just a space. */
+char *
+window_printable_flags(struct session *s, struct winlink *wl)
+{
+	char	flags[BUFSIZ];
+	int	pos;
+
+	pos = 0;
+	if (wl->flags & WINLINK_ACTIVITY)
+		flags[pos++] = '#';
+	if (wl->flags & WINLINK_BELL)
+		flags[pos++] = '!';
+	if (wl->flags & WINLINK_CONTENT)
+		flags[pos++] = '+';
+	if (wl->flags & WINLINK_SILENCE)
+		flags[pos++] = '~';
+	if (wl == s->curw)
+		flags[pos++] = '*';
+	if (wl == TAILQ_FIRST(&s->lastw))
+		flags[pos++] = '-';
+	if (pos == 0)
+		flags[pos++] = ' ';
+	flags[pos] = '\0';
+	return (xstrdup(flags));
+}
+
+/* Find pane in global tree by id. */
+struct window_pane *
+window_pane_find_by_id(u_int id)
+{
+	struct window_pane	wp;
+
+	wp.id = id;
+	return (RB_FIND(window_pane_tree, &all_window_panes, &wp));
+}
+
 struct window_pane *
 window_pane_create(struct window *w, u_int sx, u_int sy, u_int hlimit)
 {
@@ -468,6 +566,9 @@ window_pane_create(struct window *w, u_int sx, u_int sy, u_int hlimit)
 	wp = xcalloc(1, sizeof *wp);
 	wp->window = w;
 
+	wp->id = next_window_pane++;
+	RB_INSERT(window_pane_tree, &all_window_panes, wp);
+
 	wp->cmd = NULL;
 	wp->shell = NULL;
 	wp->cwd = NULL;
@@ -520,6 +621,8 @@ window_pane_destroy(struct window_pane *wp)
 		bufferevent_free(wp->pipe_event);
 	}
 
+	RB_REMOVE(window_pane_tree, &all_window_panes, wp);
+
 	if (wp->cwd != NULL)
 		xfree(wp->cwd);
 	if (wp->shell != NULL)
@@ -534,8 +637,7 @@ window_pane_spawn(struct window_pane *wp, const char *cmd, const char *shell,
     const char *cwd, struct environ *env, struct termios *tio, char **cause)
 {
 	struct winsize	 ws;
-	int		 mode;
-	char		*argv0;
+	char		*argv0, paneid[16];
 	const char	*ptr;
 	struct termios	 tio2;
 
@@ -582,6 +684,8 @@ window_pane_spawn(struct window_pane *wp, const char *cmd, const char *shell,
 
 		closefrom(STDERR_FILENO + 1);
 
+		xsnprintf(paneid, sizeof paneid, "%%%u", wp->id);
+		environ_set(env, "TMUX_PANE", paneid);
 		environ_push(env);
 
 		clear_signals(1);
@@ -608,10 +712,8 @@ window_pane_spawn(struct window_pane *wp, const char *cmd, const char *shell,
 		fatal("execl failed");
 	}
 
-	if ((mode = fcntl(wp->fd, F_GETFL)) == -1)
-		fatal("fcntl failed");
-	if (fcntl(wp->fd, F_SETFL, mode|O_NONBLOCK) == -1)
-		fatal("fcntl failed");
+	setblocking(wp->fd, 0);
+
 	wp->event = bufferevent_new(wp->fd,
 	    window_pane_read_callback, NULL, window_pane_error_callback, wp);
 	bufferevent_enable(wp->event, EV_READ|EV_WRITE);
@@ -832,7 +934,8 @@ window_pane_mouse(
 	m->y -= wp->yoff;
 
 	if (wp->mode != NULL) {
-		if (wp->mode->mouse != NULL)
+		if (wp->mode->mouse != NULL &&
+		    options_get_number(&wp->window->options, "mode-mouse"))
 			wp->mode->mouse(wp, sess, m);
 	} else if (wp->fd != -1)
 		input_mouse(wp, m);
